@@ -294,6 +294,9 @@ local function request(self, u, ex, callback)
 		if ex and ex.use_dane ~= nil then
 			use_dane = ex.use_dane;
 		end
+		if not sslctx then
+			error("Attempt to make HTTPS request but no 'sslctx' provided in options");
+		end
 	end
 
 	local http_service = basic_resolver.new(host, port_number, "tcp", { servername = req.host; use_dane = use_dane });
